import os
import json
from typing import List, Optional
from pydantic import BaseModel, parse_obj_as
from pydantic.json import pydantic_encoder


from .config import ENV_FILE_PATH 


class Env(BaseModel):
    domain_name: str
    api_key: str


class EnvironmentStorage:
    envs: Optional[List[Env]]

    def __init__(self):
        self._create_default_config_file()

    def _create_default_config_file(self):
        if not os.path.exists(ENV_FILE_PATH):
            with open(ENV_FILE_PATH,"w+") as f:
                json.dump([], f)

    def _read_file(self):
        with open(ENV_FILE_PATH, "r") as f:
            envs = json.load(f)
            self.envs = parse_obj_as(List[Env], envs)

    def get_all_envs(self):
        self._read_file()
        return self.envs

    def add_env(self, env: Env):
        with open(ENV_FILE_PATH, "r+") as f:
            envs = json.load(f)

        envs = parse_obj_as(List[Env], envs)
        env_domains = map(lambda e: e.domain_name, envs)

        with open(ENV_FILE_PATH, "w+") as f:
            if env.domain_name in env_domains:
                raise EnvAlreadyExists()
            envs.append(env)
            f.write(json.dumps(envs, default=pydantic_encoder, indent=2))

    def remove_env(self, index: int):
        with open(ENV_FILE_PATH, "r+") as f:
            envs = json.load(f)
        envs: List[Env] = parse_obj_as(List[Env], envs)

        with open(ENV_FILE_PATH, "w+") as f:
            envs.pop(index)
            f.write(json.dumps(envs, default=pydantic_encoder, indent=2))


class EnvAlreadyExists(Exception):
    ...


from shutil import which
import subprocess


def prompt(choices=None, header="", multi=False):
    fzf_options = [] 
    if header:
        fzf_options += [f"--header={header}", ]
    if multi:
        fzf_options += [f"--multi", ]


    fzf = FzfPrompt()
    return fzf.prompt(choices, fzf_options=fzf_options)

def prompt_numbered(choices,  *args, transform=None, **kwargs):
    enumerated_choices = []
    for idx, choice in enumerate(choices):
        if transform:
            choice = transform(choice)
        enumerated_choices.append(f"{idx}:{choice}")

    selected = prompt(enumerated_choices)[0]
    idx, _ = selected.split(":")
    return  choices[int(idx)]



FZF_URL = "https://github.com/junegunn/fzf"

class FzfPrompt:
    def __init__(self, executable_path=None):
        if executable_path:
            self.executable_path = executable_path
        elif not which("fzf") and not executable_path:
            raise SystemError(
                f"Cannot find 'fzf' installed on PATH. ({FZF_URL})")
        else:
            self.executable_path = "fzf"

    def prompt(self, choices=None, fzf_options=None, delimiter='\n'):
        if not fzf_options:
            fzf_options = []
        # convert lists to strings [ 1, 2, 3 ] => "1\n2\n3"
        choices_str = delimiter.join(map(str, choices))
        selection = []


        # Invoke fzf externally and write to output file

        pipe = subprocess.run([self.executable_path] + fzf_options, stdout=subprocess.PIPE, input=choices_str, text=True)
        output: str = pipe.stdout

        for line in output.splitlines():
            selection.append(line)


        return selection


from typing import Optional
import json
from dataclasses import dataclass

from .staircase_http_client import StaircaseHttpClient


@dataclass
class StaircaseEnvironment:
    domain: str
    api_key: str

    http_client: Optional[StaircaseHttpClient] = None

    def __post_init__(self):
        self.http_client = StaircaseHttpClient(env=self)



import click
import rich

from staircase.config import CONFIG_FILE_PATH
from staircase.lib.click import async_cmd


@click.command(name="file-path")
@async_cmd
async def command():
    rich.print(CONFIG_FILE_PATH)





import click

from . import set_health_check

@click.group(name="health")
def health_group():
    """
    Manage your environments.
    """
    ...

health_group.add_command(set_health_check.command)


from staircase.postman import StaircaseNotFoundBundle
from staircase.lib.sdk import StaircaseEnvironment

from .config import UserConfig


class Staircase:
    def __init__(self, config: UserConfig):
        if not config.marketplace_api_key:
            raise Exception("No marketplace_api_key found in config")
        self.market_place_env = StaircaseEnvironment(
            domain="marketplace.staircaseapi.com", api_key=config.marketplace_api_key
        )

    async def get_marketplace_onltology_representation(self):
        return await self.market_place_env.http_client.async_request(
            "marketplace/ontology/representation/raw"
        )

    async def toggle_health_check_for_product(
        self, env: StaircaseEnvironment, product_id: str, is_disabled: bool
    ):
        return await env.http_client.async_request(
            "code-health-checker/development-products",
            method="POST",
            data={"product_id": product_id, "is_disabled": is_disabled},
        )

    async def create_transaction(self, staircase_env, label):
        return await staircase_env.http_client.async_request(
            "persistence/transactions", "POST", data={"label": label}
        )

    async def get_latest_transaction_by_label(self, staircase_env, label):
        response = await staircase_env.http_client.async_request(
            f"persistence/transactions?filter=label+eq+{label}", "GET"
        )
        json_ = await response.json()
        transactions = json_["transactions"]
        if len(transactions) > 0:
            return transactions[0]["transaction_id"]
        return None

    async def create_collection(self, staircase_env, transaction_id, collection):
        return await staircase_env.http_client.async_request(
            f"persistence/transactions/{transaction_id}/collections",
            "POST",
            data={"data": collection, "metadata": {"version": 3, "validation": True}},
        )

    async def get_all_product_components(self):
        url = f"marketplace/ontology/representation/raw"
        response = await self.market_place_env.http_client.async_request(url)
        response_json = await response.json()
        products = response_json.get("products", [])

        return products

    async def get_all_representation(self):
        url = f"marketplace/ontology/representation/raw"
        response = await self.market_place_env.http_client.async_request(url)
        response_json = await response.json()
        return response_json

    async def retrieve_product_component(self, product_component_name):
        url = f"marketplace/products/{product_component_name}"
        response = await self.market_place_env.http_client.async_request(url)

        return await response.json()

    async def get_env_token(self, staircase_env):
        response = await staircase_env.http_client.async_request(
            f"environment-manager/environment"
        )
        env_data = await response.json()
        token = env_data["staircase_environment"]["environment_token"]
        return token

    async def get_latest_bundle(self, component_name, is_data=False) -> str:
        next_token = None

        
        while True:
            url = f"marketplace/{'data' if is_data else 'products'}/{component_name}/bundles"
            if next_token:
                url += f"?next_token={next_token}"

            res = await self.market_place_env.http_client.async_request(url)
            if res.status == 404:
                raise StaircaseNotFoundBundle("Recieve 404")

            if res.status == 403:
                raise StaircaseNotFoundBundle("Component name might contain slash")

            response_json = await res.json()
            bundles = response_json["bundles"]
            page = response_json["page"]

            for bundle in bundles:
                if not bundle["bundle_status"] == "Valid" or "bundle_url" not in bundle:
                    continue
                return bundle["bundle_url"]

            next_token = page.get("next_token")
            if not next_token:
                break

        raise StaircaseNotFoundBundle("Not found latest bundle with success assess")



import click

from staircase.lib.postman_http_client import PostmanClient



class Postman():
    client: PostmanClient

    def __init__(self, client: PostmanClient):
        self.client = client

    def cleanup_postman_collections(self, collection_name, workspaceid):
        response = self.client.request("collections")
        all_collections = response.json()["collections"]
        collections_to_delete = filter(
            lambda collection: collection["name"] == collection_name,
            all_collections,
        )
        for collection in collections_to_delete:
            self.client.request(
                f'collections/{collection["id"]}?workspace={workspaceid}', method="DELETE"
            )

    def get_or_create_workspace(self):
        workspace_name = "StaircaseAPI"

        response = self.client.request(
            f"workspaces",
        )

        def get_id_of_created_workspace(workspaces):
            for workspace in workspaces:
                if workspace["name"] == workspace_name:
                    return workspace["id"]

        workspace_id = get_id_of_created_workspace(response.json()["workspaces"])

        if not workspace_id:
            response = self.client.request(
                f"workspaces",
                method="POST",
                json={
                    "workspace": {
                        "name": workspace_name,
                        "description": "Generated by Staircase CLI",
                        "type": "personal",
                    }
                },
            )
            workspace_id = response.json()["workspace"]["id"]
            click.echo(f"Created new Postman workspace with id: {workspace_id}")
            return workspace_id

        click.echo(
            f"Postman workspace with name {workspace_name} exists with id: {workspace_id}"
        )

        return workspace_id

    def delete_env_by_id(self, env_id: str):
        return self.client.request(f"environments/{env_id}", "DELETE")


class StaircaseNotFoundBundle(Exception):
    ...


import logging


logger = logging.getLogger("staircase")
logger.setLevel(logging.DEBUG)

fileHandler = logging.FileHandler("staircase.log")
fileHandler.setLevel(logging.DEBUG)
logFormatter = logging.Formatter("%(asctime)s [%(levelname)-5.5s]\n%(message)s")
fileHandler.setFormatter(logFormatter)
logger.addHandler(fileHandler)



import rich
import click

from staircase.lib.postman_http_client import PostmanClient
from staircase.lib.click import async_cmd
from staircase.command_providers import (
    env_storage_provider,
    postman_provider,
)
from staircase.env_storage import EnvironmentStorage
from staircase.postman import Postman


@click.command(name="envs")
@click.option("-s", "--sync", flag_value=True)
@postman_provider()
@env_storage_provider()
@async_cmd
async def command(
    postman: Postman,
    env_storage: EnvironmentStorage,
    postman_client: PostmanClient,
    sync: bool,
    **_,
):
    """
    Import Staircase env to your Postman and use it with api product you imported.
    Creates StaircaseAPI workspace.  

    This command imports all envs that you have in `sc envs` group.

    If `-s` `--sync` flag is enabled. Envs that are in your Postman are same as `sc envs list` output.
    """
    workspaceid = postman.get_or_create_workspace()
    postman_envs = postman_client.request("environments").json()["environments"]
    envs = env_storage.get_all_envs()

    if sync:
        env_domain_names = [env.domain_name for env in envs]
        env_domain_names_to_postman_env_name = [
            get_postman_env_name_from_domain(edn) for edn in env_domain_names
        ]
        postman_envs_to_delete = filter(
            lambda p_env: p_env["name"] not in env_domain_names_to_postman_env_name,
            postman_envs,
        )
        for p_env in postman_envs_to_delete:
            postman.delete_env_by_id(p_env["id"])
            rich.print(f"[red] Delete env {p_env['name']}")


    for env in envs:
        postman_env_name = get_postman_env_name_from_domain(env.domain_name)

        envs_with_same_name = filter(
            lambda p_env: p_env["name"] == postman_env_name, postman_envs
        )
        [postman.delete_env_by_id(p_env["id"]) for p_env in envs_with_same_name]

        # create env

        postman_client.request(
            f"environments/?workspace={workspaceid}",
            "POST",
            json={
                "environment": {
                    "name": postman_env_name,
                    "values": [
                        {"key": "envUrl", "value": f"https://{env.domain_name}"},
                        {
                            "key": "apiKey",
                            "value": env.api_key,
                        },
                    ],
                }
            },
        )
        rich.print(f"[green]Imported {env.domain_name}")


def get_postman_env_name_from_domain(domain: str):
    return domain.split(".staircaseapi.com")[0]



import time


def assess(self, source_url):
    r = self.CI_ENV.assess_product.assess(source_url)
    id_ = r["assessment_id"]
    while True:
        response_body = self.CI_ENV.assess_product.check_assessment(id_)
        status = response_body.get("status")
        print(f"Status of assess: {status}")
        if status in ("IN_PROGRESS", "RUNNING"):
            time.sleep(15)
        elif status == "FAILED":
            print(f"Assess failed")
            if logs := response_body.get("logs"):
                logs = "".join(logs) if isinstance(logs, list) else logs
            print(response_body)
            return source_url
        else:
            source_url = response_body["source_url"]
            return source_url



import rich
import click

from staircase.lib.click import async_cmd
from staircase.env_storage import EnvironmentStorage 
from staircase.command_providers import env_storage_provider
from staircase.lib.fzf import prompt


@click.command(name="remove")
@env_storage_provider()
@async_cmd
async def command( env_storage: EnvironmentStorage, **_):
    envs = env_storage.get_all_envs()
    envs_to_delete = prompt([ f"{index} - {env.domain_name}:{env.api_key}" for index, env in enumerate(envs) ])
    for env_to_delete in envs_to_delete:
        index = int(env_to_delete.split('-')[0])
        env_storage.remove_env(index)

    rich.print("[green]Removed.")




import click

from . import list
from . import add
from . import remove

@click.group(name="envs")
def envs_group():
    """
    Manage your environments.
    """
    ...

envs_group.add_command(list.command)
envs_group.add_command(add.command)
envs_group.add_command(remove.command)


import asyncio

from ..staircase_env import StaircaseEnvironment


async def clone(
    ci_env: StaircaseEnvironment, *, repo: str, branch: str, github_token: str, verbose: bool = False
):
    data = {
        "repository_name": repo,
        "github_token": github_token,
        "branch": branch,
        "github_account": "staircaseapi",
    }
    r = await ci_env.http_client.async_request(
        "code/clone",
        "POST",
        data=data,
    )
    r = await r.json()

    clone_id = r["bundle_id"]
    if verbose:
        print(f"Clone id: {clone_id}")

    while True:
        response = await ci_env.http_client.async_request(f"code/clone/{clone_id}")
        response_body = await response.json()
        status = response_body.get("clone_status")
        if status in ("IN_PROGRESS", "RUNNING"):
            await asyncio.sleep(10)
        elif status == "FAILED":
            if logs := response_body.get("logs"):
                logs = "".join(logs) if isinstance(logs, list) else logs
            raise Exception()
        elif status == "SUCCEEDED":
            url: str = response_body["source_url"]
            return url
        else:
            raise Exception()



import os
import click

from .config import DATA_FOLDER, VAR_FOLDER, CONFIG_FILE_PATH

from .commands.postman import postman_group
from .commands.envs import envs_group
from .commands import config as config_
from .commands import ci
from .commands import asana
from .commands import health


def get_cli():
    cli = click.Group()
    cli.add_command(postman_group)
    cli.add_command(envs_group)
    cli.add_command(config_.group)
    cli.add_command(ci.command)
    cli.add_command(asana.group)
    cli.add_command(health.health_group)
    return cli


def init_cli():
    if not os.path.exists(DATA_FOLDER):
        os.makedirs(DATA_FOLDER)

    if not os.path.exists(VAR_FOLDER):
        os.makedirs(VAR_FOLDER)

    if not os.path.exists(CONFIG_FILE_PATH):
        CONFIG_FILE_PATH.parent.mkdir(exist_ok=True, parents=True)


    cli = get_cli()
    return cli

def launch_cli():
    cli = init_cli()
    cli()





import click
import functools as ft


from .postman import PostmanClient, Postman
from .staircase import Staircase
from .env_storage import EnvironmentStorage
from .env_manager import EnvironmentManager
from .config import get_user_cfg 



def user_config_provider():
    def decorator(f):
        @click.pass_context
        def wrapper(ctx, *args, **kwargs):
            user_config = get_user_cfg()
            return ctx.invoke(f, *args, **kwargs, user_config=user_config)

        return ft.update_wrapper(wrapper, f)

    return decorator


def staircase_provider():
    def decorator(f):
        @click.pass_context
        def wrapper(ctx, *args, **kwargs):
            user_config = get_user_cfg()
            staircase = Staircase(user_config)
            return ctx.invoke(f, *args, **kwargs, staircase=staircase)
        return ft.update_wrapper(wrapper, f)
    return decorator


def env_storage_provider():
    def decorator(f):
        @click.pass_context
        def wrapper(ctx, *args, **kwargs):
            env_storage = EnvironmentStorage()
            return ctx.invoke(f, *args, **kwargs, env_storage=env_storage)
        return ft.update_wrapper(wrapper, f)
    return decorator

def env_manager_provider():
    def decorator(f):
        @click.pass_context
        def wrapper(ctx, *args, **kwargs):
            obj = EnvironmentManager()
            return ctx.invoke(f, *args, **kwargs, env_manager=obj)
        return ft.update_wrapper(wrapper, f)
    return decorator


def postman_provider():
    def decorator(f):
        @click.pass_context
        def wrapper(ctx, *args, **kwargs):
            user_config = get_user_cfg()
            postman_client = PostmanClient(api_key=user_config.postman_api_key)
            postman = Postman(postman_client)
            return ctx.invoke(
                f, *args, **kwargs, postman_client=postman_client, postman=postman
            )

        return ft.update_wrapper(wrapper, f)

    return decorator



import rich
import click
from itertools import product
from ulid import ULID

from staircase.lib.fzf import prompt
from staircase.staircase_asana import StaircaseAsana, SEARCHES_BY_TEAM
from staircase.lib.click import async_cmd
from staircase.command_providers import user_config_provider, env_manager_provider
from staircase.config import UserConfig
import webbrowser
from staircase.command_providers import env_storage_provider, staircase_provider, EnvironmentManager
from staircase.lib.sdk.staircase_env import StaircaseEnvironment
from staircase.env_storage import EnvironmentStorage
from staircase.staircase import Staircase


@click.command(name="searches")
@click.option("--open-browser", "-o", help="Open in browser", is_flag=True)
@click.option("--save", "-s", help="Saves in persistence.", is_flag=True)
@click.option("--env", "-e", help="Target env to save")
@user_config_provider()
@env_storage_provider()
@staircase_provider()
@env_manager_provider()
@async_cmd
async def command(
    open_browser: bool,
    save: bool,
    env: str,
    user_config: UserConfig,
    env_storage: EnvironmentStorage,
    staircase: Staircase,
    env_manager: EnvironmentManager
):
    if not user_config.asana_token:
        raise Exception("No asana token in config")
    asana = StaircaseAsana(token=user_config.asana_token)

    teams = asana.get_staircase_teams()

    verbose_names = []
    searches_combinations = product(SEARCHES_BY_TEAM, teams)
    # Genearting constructed searches
    searches = map(
        lambda search_combination: search_combination[0](
            team=search_combination[1], staircase_asana=asana
        ),
        searches_combinations,
    )


    if save:
        env: Optional[StaircaseEnvironment] = env_manager.get_staircase_env(env)
        await save_searches_in_persistence(searches, env, staircase)
        
        return

    enumed_searches = list(enumerate(searches))
    for idx, search in enumed_searches:
        verbose_names.append(f"{idx}: {search.verbose_name}")

    selected = prompt(verbose_names)[0]
    idx = int(selected.split(":")[0])
    search = enumed_searches[idx][1]

    if open_browser:
        webbrowser.open(search.search_url, new=0, autoraise=True)
    else:
        rich.print(search.search_url)


async def save_searches_in_persistence(searches, env, staircase: Staircase):

    transaction_id = await staircase.get_latest_transaction_by_label(env, "asana-searches")
    if not transaction_id:
        response = await staircase.create_transaction(env, "asana-searches")
        js_ = await response.json()
        transaction_id = js_["transaction_id"]

    saved_searches = []
    for search in searches:
        try:
            saved_searches.append({
                "search_url": search.search_url,
                "search_terms": search.verbose_name,
                "@id": str(ULID()),
                "@type": "saved_search",
            })
        except Exception as e:
            print(search.verbose_name)


    collection_to_save = {"saved_searches": saved_searches}

    response = await staircase.create_collection(
        env, transaction_id, collection_to_save
    )
    print(response)



# TODO add note that this command removes collections with same names
# TODO add imported collection not as swagger but as component name
import zipfile
import os
import rich
import shutil
import yaml
import requests
import click
from typing import Union
from staircase.config import VAR_FOLDER

from staircase.lib.shutil import onerror
from staircase.lib.fzf import prompt
from staircase.lib.postman_http_client import PostmanClient
from staircase.lib.click import async_cmd

from staircase.command_providers import (
    staircase_provider,
    postman_provider,
)
from staircase.postman import Postman
from staircase.staircase import Staircase, StaircaseNotFoundBundle


@click.command(name="api")
@click.option("-c", "--component")
@click.option("-a", "--all", flag_value=True, default=False)
@click.option("-d", "--data", flag_value=True, default=False)
@staircase_provider()
@postman_provider()
@async_cmd
async def command(
    all: bool,
    data: bool,
    component: Union[str, bool],
    staircase: Staircase,
    postman: Postman,
    postman_client: PostmanClient,
):
    """
    Import Staircase product to your Postman and use it with environment you imported.
    Creates StaircaseAPI workspace.

    """

    components = (await staircase.get_all_representation())["components"]
    if not components:
        raise Exception("No components found")
    components_names = map(lambda e: e["component_name"], components)

    if not all and not component:
        components_names = prompt(
            components_names,
            header="Select your component name | Select multiple via TAB",
            multi=True,
        )

    workspaceid = postman.get_or_create_workspace()

    for component_name in components_names:
        click.echo(f"Importing component {component_name}...")
        os.chdir(VAR_FOLDER)

        try:
            bundle_url = await staircase.get_latest_bundle(component_name, data)
        except StaircaseNotFoundBundle:
            rich.print(
                f"[red]Not found latest for component {component_name} in marketplace"
            )
            continue

        click.echo(f"Downloading bundle...")
        click.echo(bundle_url)
        r = requests.get(bundle_url, allow_redirects=True)
        click.echo(f"Downloaded")
        with open(VAR_FOLDER / "bundle.zip", "wb") as f:
            f.write(r.content)

        bundle_folder_path = VAR_FOLDER / "bundle"

        with zipfile.ZipFile(VAR_FOLDER / "bundle.zip", "r") as zip_ref:
            zip_ref.extractall(bundle_folder_path)

        swagger_file_path = bundle_folder_path / "docs" / "swagger.yml"

        if not os.path.exists(swagger_file_path):
            click.echo("Not found swagger file in component repo, nothing to import.")
            continue

        with open(swagger_file_path, "r", encoding="utf-8") as f:
            swagger_json = yaml.safe_load(f)
            swagger_json["info"]["title"] = component_name

        postman.cleanup_postman_collections(component_name, workspaceid)
        shutil.rmtree(bundle_folder_path, onerror=onerror)

        response = postman_client.request(
            f"import/openapi?workspace={workspaceid}",
            method="POST",
            json={"type": "json", "input": swagger_json},
        )

        response = response.json()
        collection_id = response["collections"][0]["id"]
        if len(response["collections"]) > 1:
            raise Exception("You can not have more than 1 collection imported. File CR")

        response = postman_client.request(
            f"collections/{collection_id}?workspace={workspaceid}"
        )
        collection_create_body = response.json()
        base_url = collection_create_body["collection"]["variable"][0]["value"]
        prefix_len = len("https://documentation.staircaseapi.com/")
        base_path = base_url[prefix_len:]

        collection_create_body["collection"]["variable"][0]["value"] = (
            "{{envUrl}}/" + base_path
        )
        # del collection_create_body["collection"]["variable"][0]["type"]

        postman_client.request(
            f"collections/{collection_id}?workspace={workspaceid}", method="DELETE"
        )
        postman_client.request(
            f"collections?workspace={workspaceid}",
            method="POST",
            json=collection_create_body,
        )
        rich.print(f"[green]{component_name} imported successfully")



import os 

def onerror(func, path, exc_info):
    """
    Error handler for ``shutil.rmtree``.

    If the error is due to an access error (read only file)
    it attempts to add write permission and then retries.

    If the error is for another reason it re-raises the error.
    
    Usage : ``shutil.rmtree(path, onerror=onerror)``
    """
    import stat
    # Is the error an access error?
    if not os.access(path, os.W_OK):
        os.chmod(path, stat.S_IWUSR)
        func(path)
    else:
        raise


import json
import rich
import click
from pydantic.json import pydantic_encoder

from staircase.lib.click import async_cmd
from staircase.env_storage import EnvironmentStorage
from staircase.command_providers import env_storage_provider


@click.command(name="list")
@env_storage_provider()
@async_cmd
async def command( env_storage: EnvironmentStorage, **_):
    rich.print_json(json.dumps(env_storage.get_all_envs(), default=pydantic_encoder))



import rich
import os
import json
from pydantic import BaseModel
from pathlib import Path
from typing import Optional


STAIRCASE_FOLDER = Path.home() / '.staircase'
DATA_FOLDER =  STAIRCASE_FOLDER / "data"
VAR_FOLDER = STAIRCASE_FOLDER / "var"
CONFIG_FILE_PATH = DATA_FOLDER / "config.json"
ENV_FILE_PATH = DATA_FOLDER / "staircase_envs.json"

class UserConfig(BaseModel):
    postman_api_key: Optional[str]
    marketplace_api_key: Optional[str]
    github_token: Optional[str]
    asana_token: Optional[str]


def get_user_cfg() -> UserConfig:
    if not os.path.exists(CONFIG_FILE_PATH):
        return UserConfig(
            postman_api_key=None,
            marketplace_api_key=None,
            github_token=None,
            asana_token=None
        )
    
    with open(CONFIG_FILE_PATH, "r") as f:
        try:
            config_file = json.load(f)
            user_config = UserConfig.parse_obj(config_file)
            return user_config
        except json.JSONDecodeError:
            rich.print("[red]Config is not valid, taking default values.")
            return UserConfig(
                postman_api_key=None,
                marketplace_api_key=None,
                github_token=None,
                asana_token=None
            )


def write_user_cfg(user_cfg: UserConfig):
    with open(CONFIG_FILE_PATH, "w+") as f:
        json.dump(user_cfg.dict(), f)



import requests
from dataclasses import dataclass

@dataclass
class PostmanClient:
    api_key: str

    def request(self, url, method="GET", data=None, json=None):
        url = f"https://api.getpostman.com/{url}"

        headers = {
        'Content-Type': 'application/json',
        'X-API-Key': self.api_key
        }

        response = requests.request(method, url, headers=headers, json=json, data = data)
        return response


import sys
from typing import Literal
import time

from ..staircase_env import StaircaseEnvironment


async def build(
    ci_env: StaircaseEnvironment,
    bundle_url,
    type: Literal["service", "data", "frontend"] = "service",
    env_variables: dict = None,
    verbose: bool = False,
    architecture: Literal["X86_64", "ARM64"] = "X86_64",
) -> str:
    if not env_variables:
        env_variables = {}

    if type == "service":
        url = "infra-builder/builds"
    elif type == "data":
        url = "infra-builder/data"
    elif type == "frontend":
        url = "infra-builder/frontend"
    elif type == "chat":
        url = "infra-builder/chat"

    try:
        data = {"source_url": bundle_url}
        if env_variables:
            data["env_variables"] = env_variables
        data["architecture"] = architecture
        r = await ci_env.http_client.async_request(url, "POST", data=data)
    except Exception as e:
        print(e.message)
        raise e
    r = await r.json()
    id_ = r["build_id"]
    if verbose:
        print(f"Build id: {id_}")

    while True:
        response = await ci_env.http_client.async_request(f"{url}/{id_}")
        response_body = await response.json()

        status = response_body.get("status")
        if status in ("IN_PROGRESS", "RUNNING"):
            time.sleep(15)
        elif status == "FAILED":
            if logs := response_body.get("logs"):
                logs = "".join(logs) if isinstance(logs, list) else logs
            raise BuildFailed(logs)
        else:
            url = response_body["artifacts_url"]
            return url


class BuildFailed(Exception):
    ...



import json
from typing import Optional 
from staircase.lib.fzf import prompt
from .staircase import Staircase

async def select_product_git_repo(staircase: Staircase, prompt_header: str) -> Optional[str]:
    products = await staircase.get_all_product_components()

    product_names = list(map(lambda e: e["product_name"], products))
    product_names_selected = prompt(product_names, header=prompt_header)
    if not product_names_selected:
        return None
    product_name_selected = product_names_selected[0]
    
    product_data = await staircase.retrieve_product_component(product_name_selected)
    repo_url = product_data.get("bundle_meta")['service-code']['repository_url']
    repo_url = repo_url.split("StaircaseAPI/")[1]

    return repo_url




def select_env_domain():
    with open("data/staircase_envs.json") as fp_envs:
        envs = json.load(fp_envs)
        env_domains = list(map(lambda e: e["domain_name"], envs))
        selected = prompt(env_domains)
        if not selected:
            return None
        return selected[0]



from typing import Optional

from .env_storage import EnvironmentStorage
from .lib.sdk import StaircaseEnvironment


class EnvironmentManager:

    def __init__(self):
        self.env_storage = EnvironmentStorage()

    def get_staircase_env(self, domain: str) -> Optional[StaircaseEnvironment]:
        envs = self.env_storage.get_all_envs()
        if not envs:
            return
        envs = list(filter(lambda e: e.domain_name == domain, envs))
        if not envs:
            return
        env = envs[0]
        return StaircaseEnvironment(domain=env.domain_name, api_key=env.api_key)



import time


def test(self, source_url):
    r = self.CI_ENV.test_product.test_ui(source_url)
    id_ = r["test_suites_invoked"][0]["uiTests"]

    while True:
        response_body = self.CI_ENV.test_product.check(id_)
        status = response_body.get("status")
        if status in ("IN_PROGRESS", "RUNNING"):
            time.sleep(15)
        elif status == "FAILED":
            print(response_body)
            return
        else:
            print(response_body)
            return



import asana
from typing import Any
from itertools import product
from dataclasses import dataclass
from staircase.lib.fzf import prompt





class StaircaseAsana:
    """Incapsulates interaction with asana"""

    STAIRCASE_WORKSPACE_NAME = "Staircase"
    STAIRCASE_TEAMS = ["moore", "little", "bushnell", "torvalds", "cerf"]

    def __init__(self, token):
        self.client = self.get_client(token)
        # Disable warnings
        self.client.LOG_ASANA_CHANGE_WARNINGS = False

    def get_client(self, token):
        client = asana.Client.access_token(token)
        return client

    def get_staircase_workspace_id(self):
        staircase_ws = list(
            filter(
                lambda e: e["name"] == self.STAIRCASE_WORKSPACE_NAME,
                self.client.workspaces.get_workspaces(),
            )
        )[0]
        return staircase_ws["gid"]

    def get_staircase_teams(self):
        staircase_ws_id = self.get_staircase_workspace_id()
        all_teams = self.client.teams.get_teams_for_workspace(staircase_ws_id)
        staircase_teams = list(filter(lambda team: team['name'] in self.STAIRCASE_TEAMS, all_teams))
        return staircase_teams


    def get_projects_for_team(self, team_name):
        staircase_team = self.get_staircase_team_by_name(team_name)
        projects = self.client.projects.get_projects_for_team(staircase_team["gid"])
        return projects

    def get_staircase_team_by_name(self, team_name):
        staircase_ws_id = self.get_staircase_workspace_id()
        staircase_teams = list(
            self.client.teams.get_teams_for_workspace(staircase_ws_id)
        )
        for team in staircase_teams:
            if team["name"] == team_name:
                return team

    def get_sections_for_project(self, project_id: str):
        sections = list(self.client.sections.get_sections_for_project(project_id))
        return sections


    # def get_staircase_projects(self):
    #     staircase_ws_id = self._get_staircase_workspace_id(self.client)
    #     staircase_projects = list(
    #         self.client.projects.get_projects_for_workspace(staircase_ws_id)
    #     )
    #     return staircase_projects

    # def get_staircase_projects_by_team(self):
    #     staircase_teams = self.get_staircase_teams(self.client)

    #     staircase_projects_by_team = {}
    #     for team in staircase_teams:
    #         projects = list(self.client.projects.get_projects_for_team(team["gid"]))
    #         staircase_projects_by_team[team["name"]] = projects
    #     return staircase_projects_by_team


@dataclass
class BaseSearchByTeam:
    staircase_asana: StaircaseAsana
    team: Any  # Asana team

    @property
    def verbose_name(self):
        team_name = self.team["name"].capitalize()
        return f"{team_name} {self.base_name}"

    def get_id_of_company_team_name(self):
        team_name = self.team["name"]

        ws_id = self.staircase_asana.get_staircase_workspace_id()
        tags = list(self.staircase_asana.client.tags.find_by_workspace(ws_id))
        for tag in tags:
            if tag['name'] == f"company: {team_name}":
                return tag['gid']

    def create_query_params_for_projects(self, section_name):
        team_name = self.team["name"]

        query_parameter = ""

        projects = list(self.staircase_asana.get_projects_for_team(team_name))
        for i, project in enumerate(projects):
            proj_id = project["gid"]
            query_parameter += proj_id

            if section_name:
                sections = self.staircase_asana.get_sections_for_project(project["gid"])
                section = [
                    section for section in sections if section["name"] == section_name
                ][0]
                section_id = section["gid"]
                query_parameter += f"_column_{section_id}"

            is_last_project = i + 1 == len(projects)
            if not is_last_project:
                query_parameter += "~"

        return query_parameter


class TicketsInboxSearch(BaseSearchByTeam):
    base_name = "Tickets Inbox"

    @property
    def search_url(self):
        projects_query = self.create_query_params_for_projects(section_name="Tickets")

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "completion=incomplete&"
            "milestone=is_not_milestone&"
            "subtask=is_not_subtask&"
            "not_tags.ids=1202112126062384~1203874644775198&"
            f"any_projects.ids={projects_query}"
        )


class TicketsGroomedSearch(BaseSearchByTeam):
    base_name = "Tickets Groomed"

    @property
    def search_url(self):
        projects_query = self.create_query_params_for_projects(section_name="Tickets")

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "completion=incomplete&"
            "milestone=is_not_milestone&"
            "subtask=is_not_subtask&"
            "all_tags.ids=1202112126062384&"
            f"any_projects.ids={projects_query}"
        )


class BacklogSearch(BaseSearchByTeam):
    base_name = "Backlog"

    @property
    def search_url(self):
        projects_query = self.create_query_params_for_projects(section_name="Backlog")

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "completion=incomplete&"
            "milestone=is_milestone&"
            "any_tags.ids=1203326207155206&"
            f"any_projects.ids={projects_query}"
        )


class NextSearch(BaseSearchByTeam):
    base_name = "Next"

    @property
    def search_url(self):
        projects_query = self.create_query_params_for_projects(section_name="Next")

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "completion=incomplete&"
            "milestone=is_milestone&"
            "any_tags.ids=1202112126062384&"
            f"any_projects.ids={projects_query}"
        )


class InProgressSearch(BaseSearchByTeam):
    base_name = "In Progress"

    @property
    def search_url(self):
        projects_query = self.create_query_params_for_projects(
            section_name="In Progress"
        )

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "completion=incomplete&"
            "milestone=is_milestone&"
            "any_tags.ids=1202112126062384&"
            f"any_projects.ids={projects_query}"
        )


class BlockedSearch(BaseSearchByTeam):
    base_name = "Blocked"

    @property
    def search_url(self):
        projects_query = self.create_query_params_for_projects()

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "completion=incomplete&"
            "milestone=is_milestone&"
            "any_tags.ids=1199657336606014&"
            "not_tags.ids=1203874644775198&"
            f"any_projects.ids={projects_query}"
        )


class DeliveryAllSearch(BaseSearchByTeam):

    @property
    def verbose_name(self):
        team_name = self.team["name"].capitalize()
        return f" Delivery {team_name}: All"

    @property
    def search_url(self):
        company_team_tag_id = self.get_id_of_company_team_name()

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "milestone=is_milestone&"
            f"all_tags.ids=1203332567269502~{company_team_tag_id}&"
            "any_projects.ids=1203875233546411~1203844243602944~1204284245172775~1203877463529063"
        )

class DeliveryBacklogSearch(BaseSearchByTeam):

    @property
    def verbose_name(self):
        team_name = self.team["name"].capitalize()
        return f" Delivery {team_name}: Backlog"

    @property
    def search_url(self):
        company_team_tag_id = self.get_id_of_company_team_name()

        return (
            "https://app.asana.com/0/search?"
            "sort=due_date&"
            "milestone=is_milestone&"
            f"all_tags.ids=1203332567269502~{company_team_tag_id}&"
            "any_projects.ids=1203875233546411_column_1203929848027686~1203844243602944_column_1203929845423949~1204284245172775_column_1204284245172776~1203877463529063_column_1203929848027689"
        )
class DeliveryNextSearch(BaseSearchByTeam):
    @property
    def verbose_name(self):
        team_name = self.team["name"].capitalize()
        return f" Delivery {team_name}: Next"

    @property
    def search_url(self):
        company_team_tag_id = self.get_id_of_company_team_name()

        return (
            "https://app.asana.com/0/search?sort=due_date&"
            "milestone=is_milestone&"
            f"all_tags.ids=1203332567269502~{company_team_tag_id}&"
            "any_projects.ids=1203875233546411_column_1203929848027688~1203844243602944_column_1203929845423951~1204284245172775_column_1204284245172780~1203877463529063_column_1203929848027691"
        )


# Seaches by team name
SEARCHES_BY_TEAM = [
    TicketsInboxSearch,
    TicketsGroomedSearch,
    BacklogSearch,
    NextSearch,
    InProgressSearch,
    # BlockedSearch,
    DeliveryAllSearch,
    DeliveryBacklogSearch,
    DeliveryNextSearch
]

if __name__ == "__main__":
    asana = StaircaseAsana()

    teams = asana.get_staircase_teams()
    teams_names = map(lambda e: e["name"], teams)
    print(teams_names)





from rich.prompt import Prompt
import click

from staircase.lib.click import async_cmd
from staircase.config import write_user_cfg, get_user_cfg


@click.command(name="setup")
@async_cmd
async def command():
    """
    Easy setup of all variables.
    """
    user_config = get_user_cfg()
    user_config.postman_api_key = Prompt.ask("Postman api key", default=user_config.postman_api_key)
    user_config.marketplace_api_key = Prompt.ask("Marketplace api key", default=user_config.marketplace_api_key)
    user_config.github_token = Prompt.ask("Github token. With authorized github cli", default=user_config.github_token)
    user_config.asana_token = Prompt.ask("Asana token", default=user_config.asana_token)
    write_user_cfg(user_config)



from typing import Optional
import rich
import subprocess
import click
from staircase.config import UserConfig
from staircase.lib.click import async_cmd

from staircase.command_providers import (
    env_manager_provider,
    staircase_provider,
    user_config_provider,
    env_storage_provider,
    EnvironmentStorage,
)
from staircase.env_manager import EnvironmentManager

from staircase.lib.fzf import prompt
from staircase.lib.click import async_cmd
from staircase.lib.sdk.staircase_env import StaircaseEnvironment
from staircase.staircase import Staircase
from staircase.lib.sdk.ci import clone, build, deploy
from staircase.user_prompts import select_product_git_repo

DEFAULT_CI_ENV = "marketplace.staircaseapi.com"


@click.command(
    "ci",
)
@click.argument("actions", type=click.Choice(["clone", "build", "deploy"]), nargs=-1)
@click.option(
    "--bundle-url",
    "-u",
    help="Url of bundle for any step you start your ci from. It will be passed correct to any ci step.",
)
@click.option(
    "--product",
    "-p",
    help="Product name in marketplace. If you dont specify clone, it will be used to get start bundle",
)
@click.option("--repo", "-r", help="Git repo name in Staircase")
@click.option(
    "--type",
    help="Type of bundle can be either service, data, chat or frontend",
    default="service",
    type=click.Choice(["service", "data", "frontend", "chat"]),
)
@click.option(
    "--ci-env",
    "-ci",
    help="Environment where ci products will be called. Ex. mydomain.staircaseapi.com",
)
@click.option("--branch", "-b", help="Branch name of git clone")
@click.option("--verbose", "-v", help="Verbose output", is_flag=True)
@click.option(
    "--target",
    "-t",
    help="Target domain of environment to be deployed. Ex. mydomain.staircaseapi.com",
)
@click.option(
    "--arch",
    help="Arch param of build product",
    default="X86_64",
    type=click.Choice(["X86_64", "ARM64"]),
)
@click.option("--build_env", "-be", help="Build env vars", multiple=True)
@env_manager_provider()
@staircase_provider()
@user_config_provider()
@env_storage_provider()
@async_cmd
async def command(
    *,
    actions,
    bundle_url,
    target,
    type,
    product,
    branch,
    repo,
    build_env,
    arch,
    ci_env: str,
    verbose: bool,
    staircase: Staircase,
    env_manager: EnvironmentManager,
    env_storage: EnvironmentStorage,
    user_config: UserConfig,
):
    """
    Setup your pipeline with clone, assess, test, deploy, build products.

    You can specify steps as arguments, order is not changable. Default order is [clone, build, assess, deploy, test]. If you dont specify any actions if will be [clone, build, deploy]
    """
    if not actions:
        actions = ["clone", "build", "deploy"]

    if not ci_env:
        envs = env_storage.get_all_envs()
        env_names = [env.domain_name for env in envs]
        selected_env = prompt(
            env_names, header="Select ci-env, default = marketplace.staircaseapi.com..."
        )
        if selected_env:
            ci_env = selected_env[0]
        else:
            ci_env = DEFAULT_CI_ENV

    ci_env: Optional[StaircaseEnvironment] = env_manager.get_staircase_env(ci_env)
    if not ci_env:
        raise click.ClickException("CI env is not saved to your envs.")

    # Getting args
    if not user_config.github_token:
        raise click.ClickException("Setup github_token in your config")

    if "clone" in actions:
        if not repo:
            repo = await select_product_git_repo(
                staircase=staircase,
                prompt_header="Select product repo to clone from...",
            )
            if not repo:
                raise click.ClickException("You did not specify any repo")

        if not branch:
            pipe = subprocess.run(
                [
                    "git",
                    "ls-remote",
                    "--heads",
                    f"git@github.com:StaircaseAPI/{repo}.git",
                ],
                stdout=subprocess.PIPE,
                text=True,
            )
            branches = []
            for line in pipe.stdout.splitlines():
                branch = line.split("refs/heads/")[1]
                branches.append(branch)

            branches_selected = prompt(branches, header="Select your branch")

            if not branches_selected:
                raise click.ClickException("You did not specify any branch")

            branch = branches_selected[0]

    if "deploy" in actions and not target:
        envs = env_storage.get_all_envs()
        env_names = [env.domain_name for env in envs]
        selected_env = prompt(env_names, header="Select env to deploy...")
        if selected_env:
            target = selected_env[0]
        if not target:
            raise click.ClickException("Not target env for deploy specified")

    if ["build", "assess", "test"] in actions:
        if not "clone" in actions or not bundle_url:
            raise click.ClickException(
                "You did not specify clone action or bundle_url to start from"
            )

    if (
        "deploy" in actions
        and not bundle_url
        and not ["build", "assess", "test", "clone"]
    ):
        product_names = [
            product["product_name"]
            for product in await staircase.get_all_product_components()
        ]
        products = prompt(product_names)
        if not products:
            raise click.ClickException(
                "No product specified and no bundle_url and no previous action step like clone. Nothing to deploy"
            )
        product_data = staircase.retrieve_product_component(product)
        bundle_url = product_data.get("bundle_url")
        if not bundle_url:
            raise click.ClickException(
                "Selected product does not contain bundle_url in marketplace."
            )

    # Flow
    if verbose:
        rich.print(f"CI_ENV: {ci_env.domain}")
    if "clone" in actions:
        if verbose:
            rich.print(f"Repo: StaircaseCLI/{repo}.git")
            rich.print(f"Branch: {branch}")
            rich.print(f"Cloning...")
        bundle_url = await clone(
            ci_env,
            repo=repo,
            branch=branch,
            github_token=user_config.github_token,
            verbose=verbose,
        )
        if verbose:
            rich.print(f"[green]Cloned.")
            rich.print(bundle_url)

    if "build" in actions:
        if verbose:
            rich.print(f"Building...")

        env_variables = {}

        for env_var in build_env:
            key, value = env_var.split("=")
            env_variables[key] = value

        bundle_url = await build(
            ci_env=ci_env,
            bundle_url=bundle_url,
            type=type,
            env_variables=env_variables,
            verbose=verbose,
            architecture=arch,
        )
        if verbose:
            rich.print(f"[green]Built.")
            rich.print(bundle_url)

    if "deploy" in actions:
        if verbose:
            if product:
                rich.print(f"Deploying bundle from marketplace for product: {product}")
            rich.print(f"Deploying to env: {target}")
            rich.print(f"Deploying...")

        to_env = env_manager.get_staircase_env(target)
        if not to_env:
            raise click.ClickException("You have no such api in your envs saved.")
        env_token = await staircase.get_env_token(to_env)

        await deploy(
            ci_env=ci_env, url=bundle_url, env_token=env_token, verbose=verbose
        )
        if verbose:
            rich.print(f"[green]Deployed.")



import rich
import click

from staircase.lib.click import async_cmd
from staircase.env_storage import EnvironmentStorage, Env
from staircase.command_providers import  env_storage_provider


@click.command(name="add")
@click.option("--domain", prompt="Env domain name")
@click.option("--api-key", prompt="Api key")
@env_storage_provider()
@async_cmd
async def command( env_storage: EnvironmentStorage, domain: str, api_key: str, **_):
    env = Env(domain_name=domain, api_key=api_key)
    env_storage.add_env(env)
    rich.print("[green]Imported.")



from typing import Optional
import rich
import subprocess
import click
import click
from staircase.config import UserConfig
from staircase.lib.click import async_cmd

from staircase.command_providers import (
    env_manager_provider,
    staircase_provider,
    user_config_provider,
    env_storage_provider,
    EnvironmentStorage,
)
from staircase.env_manager import EnvironmentManager

from staircase.lib.fzf import prompt_numbered, prompt
from staircase.lib.click import async_cmd
from staircase.lib.sdk.staircase_env import StaircaseEnvironment
from staircase.staircase import Staircase
from staircase.lib.sdk.ci import clone, build, deploy
from staircase.user_prompts import select_product_git_repo


@click.command(
    "set_health_check",
)
@click.option(
    "--product",
    "-p",
    help="Product name in marketplace.",
)
@click.option(
    "--target",
    "-t",
    help="Target domain of environment.",
)
@click.option("--disable", "-d", help="By default it enables.", flag_value=True)
@env_manager_provider()
@staircase_provider()
@user_config_provider()
@env_storage_provider()
@async_cmd
async def command(
    *,
    product,
    target,
    disable: bool,
    staircase: Staircase,
    env_manager: EnvironmentManager,
    env_storage: EnvironmentStorage,
    user_config: UserConfig,
):
    products = await staircase.get_all_product_components()
    if not product:
        product = prompt_numbered(products, transform=lambda e: e["product_name"])

        envs = env_storage.get_all_envs()
        env_names = [env.domain_name for env in envs]
        selected_env = prompt(env_names, header="Select env to deploy...")
        if selected_env:
            target = selected_env[0]

    env: Optional[StaircaseEnvironment] = env_manager.get_staircase_env(target)
    is_disabled = True if disable else False
    response = await staircase.toggle_health_check_for_product(
        env, product["product_id"], is_disabled
    )
    print(await response.json())



from dataclasses import dataclass
import aiohttp
import time

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .staircase_env import StaircaseEnvironment


def fix_retry(times=10):
    """Use to fight with random 403 error"""

    def decorator(func):
        async def newfn(*args, **kwargs):
            attempt = 0
            while attempt < times:
                response = None
                try:
                    response = await func(*args, **kwargs)
                    if response.status != 403:
                        return response
                except aiohttp.ClientResponseError as e:
                    status = e.status
                    if status != 403:
                        raise e
                except aiohttp.ServerDisconnectedError as e:
                    ...

                attempt += 1
                time.sleep(1)

            response = await func(*args, **kwargs)
            return response

        return newfn

    return decorator


@dataclass
class StaircaseHttpClient:
    env: "StaircaseEnvironment"

    @fix_retry()
    async def async_request(
        self, url, method="GET", data=None, raise_for_status=True
    ) -> aiohttp.ClientResponse:
        async with aiohttp.ClientSession() as session:

            async with session.request(
                method,
                f"https://{self.env.domain}/{url}",
                json=data,
                headers={"x-api-key": self.env.api_key},
                raise_for_status=raise_for_status,
            ) as response:
                await response.text()
                return response



import time

from ..staircase_env import StaircaseEnvironment

def deploy_data(url, to_env, ci_env):
    check_deploy = ci_env.deployer_product.check_deploy_data
    r = ci_env.deployer_product.deploy_data(url, to_env)
    id_ = r["bundle_id"]

    while True:
        response_body = check_deploy(id_)
        status = response_body.get("deploy_status")

        if status in ("IN_PROGRESS", "RUNNING"):
            time.sleep(10)
        elif status == "FAILED":
            if logs := response_body.get("logs"):
                logs = "".join(logs) if isinstance(logs, list) else logs
            print("Error deploy")
            raise Exception(response_body)
        else:
            return


async def deploy(url, ci_env: StaircaseEnvironment, env_token: str, verbose: bool = False):

    data = {
        "artifacts_url": url,
        "environment_token": env_token
    }
    r = await ci_env.http_client.async_request( "infra-deployer/deployments", "POST", data)
    r = await r.json()
    id_ = r["bundle_id"]
    if verbose:
        print(f"Deploy id: {id_}")


    while True:
        r = await ci_env.http_client.async_request( f"infra-deployer/deployments/{id_}")
        response_body = await r.json()
        status = response_body.get("deploy_status")

        if status in ("IN_PROGRESS", "RUNNING"):
            time.sleep(10)
        elif status == "FAILED":
            if logs := response_body.get("logs"):
                logs = "".join(logs) if isinstance(logs, list) else logs
            raise Exception(response_body)
        elif status == "SUCCEEDED":
            return
        else:
            raise Exception("Unknown status")
